

class OperationPriority:
    """Priority options of a operation strategy in a multiuse case
        VERY_HIGH = 0
        HIGH = 1
        MEDIUM = 2
        LOW = 3
        VERY_LOW = 4"""
    VERY_HIGH = 0
    HIGH = 1
    MEDIUM = 2
    LOW = 3
    VERY_LOW = 4
